# This script will update CloudEndure Blueprints for a specific Project.
# You can use this to define the Blueprints for Production Failover or Test Failover.
# Recommend Python 3 to run this script.
# To run this command to run the script.
# Python CE-Updates-Blueprints.py --userapitoken <API TOKEN> --projectname <PROJECT NAME> --mode <prod or test> --configfile ce-config.yml
from __future__ import print_function
import argparse
import sys
import requests
import json
import os
import yaml

##############
# Parameters #
##############
HOST = 'https://console.cloudendure.com'
headers = {'Content-Type': 'application/json'}
session = {}
endpoint = '/api/latest/{}'

###############
# API Log in  #
###############
def login(userapitoken, endpoint):
    login_data = {'userApiToken': userapitoken}
    r = requests.post(HOST + endpoint.format('login'),
                  data=json.dumps(login_data), headers=headers)
    if r.status_code != 200 and r.status_code != 307:
        if r.status_code == 401:
            print("ERROR: The login credentials provided cannot be authenticated....")
        elif r.status_code == 402:
            print("ERROR: There is no active license configured for this account....")
        elif r.status_code == 403:
            print("ERROR: Access Denied.  Verify you are using the correct API Token....")
        elif r.status_code == 429:
            print("ERROR: Authentication failure limit has been reached. The service will become available for additional requests after a timeout....")
        sys.exit(1)

    # check if need to use a different API entry point
    if r.history:
        endpoint = '/' + '/'.join(r.url.split('/')[3:-1]) + '/{}'
        r = requests.post(HOST + endpoint.format('login'),
                      data=json.dumps(login_data), headers=headers)
                      
    session['session'] = r.cookies['session']
    try:
       headers['X-XSRF-TOKEN'] = r.cookies['XSRF-TOKEN']
    except:
       pass

#############
# Arguments #
#############
def main(arguments):

    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--userapitoken', required=True)
    parser.add_argument('--projectname', required=True)
    parser.add_argument('--mode', choices=['prod','test'], required=True)
    parser.add_argument('--configfile', required=True)
    args = parser.parse_args(arguments)

################
# Login to CE  #
################

    print("************************")
    print("* Login to CloudEndure *")
    print("************************")
    login(args.userapitoken, endpoint)

    print("****Connected to CloudEndure Console****")

###############
# Get Project #
###############
    r = requests.get(HOST + endpoint.format('projects'), headers = headers, cookies = session)
    if r.status_code != 200:
        print("ERROR: Failed to fetch the project....")
        sys.exit(2)
    try:
        # Get Project ID
        projects = json.loads(r.content)["items"]
        project_exist = False
        for project in projects:
            if project["name"] == args.projectname:
               project_id = project["id"]
               project_exist = True
        if project_exist == False:
            print("ERROR: Project Name does not exist....")
            sys.exit(3)

################
# Get Machines #
################
        m = requests.get(HOST + endpoint.format('projects/{}/machines').format(project_id), headers=headers, cookies=session)
        if "sourceProperties" not in m.text:
            print("ERROR: Failed to fetch the machines....")
            sys.exit(3)
        machinelist = {}
        machines = json.loads(m.text)["items"]
        for machine in machines:
            print('Machine name:{}, Machine ID:{}'.format(machine['sourceProperties']['name'], machine['id']))
            machinelist[machine['id']] = machine['sourceProperties']['name']
    except:
        print(sys.exc_info())
        sys.exit(5)

#####################
# Update Blueprints #
#####################
    print("****Updating CloudEndure Blueprints****")
    with open(os.path.join(sys.path[0], args.configfile), 'r') as ymlfile:
            config = yaml.load(ymlfile, Loader=yaml.FullLoader)
    try:
        b = requests.get(HOST + endpoint.format('projects/{}/blueprints').format(project_id), headers=headers, cookies=session)
        for blueprint in json.loads(b.content)["items"]:
            machineName = machinelist[blueprint["machineId"]]
            for i in range(1, config["project"]["machinecount"]+1):
                index = "machine" + str(i)
                if config[index]["machineName"] == machineName:
                    url = endpoint.format('projects/{}/blueprints/').format(project_id) + blueprint['id']
                    blueprint["instanceType"] = config[index]["instanceType"]
                    blueprint["tenancy"] = config[index]["tenancy"]
                    if config[index]["iamRole"].lower() != "none":
                        blueprint["iamRole"] = config[index]["iamRole"]
                    if args.mode == "Prod":
                        blueprint["subnetIDs"] = config[index]["prodsubnetIDs"]
                        blueprint["securityGroupIDs"] = config[index]["prodsecuritygroupIDs"]
                        blueprint["privateIPAction"] = config[index]["prodprivateIPAction"]
                        blueprint["publicIPAction"] = config[index]["publicIPAction"]
                        blueprint["runAfterLaunch"] = config[index]["runAfterLaunch"]
                        if config[index]["proddiskType"] == "PROVISIONED_SSD" or "PROVISIONED_IO2":
                            for disk in blueprint["disks"]:
                                disk["type"] = config[index]["proddiskType"]
                                disk["iops"] = config[index]['proddiskIops']
                        elif config[index]["proddiskType"] == "GP3":
                            for disk in blueprint["disks"]:
                                disk["type"] = "GP3"
                                disk["iops"] = config[index]['proddiskIops']
                                disk["throughput"] = config[index]['proddiskThroughput']
                        else:
                            for disk in blueprint["disks"]:
                                disk["type"] = config[index]["proddiskType"]
                        tags = []
                        for i in range(1, config[index]["prodtags"]["count"]+1):
                            keytag = "key" + str(i)
                            valuetag = "value" + str(i)
                            tag = {"key":config[index]["prodtags"][keytag], "value":config[index]["prodtags"][valuetag]}
                            tags.append(tag)
                        blueprint["tags"] = tags
                    else:
                        if args.mode == "Test":
                            blueprint["subnetIDs"] = config[index]["testsubnetIDs"]
                            blueprint["securityGroupIDs"] = config[index]["testsecuritygroupIDs"]
                            blueprint["privateIPAction"] = config[index]["testprivateIPAction"]
                            blueprint["publicIPAction"] = config[index]["publicIPAction"]
                            blueprint["runAfterLaunch"] = config[index]["runAfterLaunch"]
                            if config[index]["testdiskType"] == "PROVISIONED_SSD" or "PROVISIONED_IO2":
                                for disk in blueprint["disks"]:
                                    disk["type"] = config[index]["testdiskType"]
                                    disk["iops"] = config[index]['testdiskIops']
                            elif config[index]["testdiskType"] == "GP3":
                                for disk in blueprint["disks"]:
                                    disk["type"] = "GP3"
                                    disk["iops"] = config[index]['testdiskIops']
                                    disk["throughput"] = config[index]['testdiskThroughput']
                        else:
                            for disk in blueprint["disks"]:
                                disk["type"] = config[index]["testdiskType"]
                        tags = []
                        for i in range(1, config[index]["testtags"]["count"]+1):
                            keytag = "key" + str(i)
                            valuetag = "value" + str(i)
                            tag = {"key":config[index]["testtags"][keytag], "value":config[index]["testtags"][valuetag]}
                            tags.append(tag)
                        blueprint["tags"] = tags
                        #else:
                            #print("ERROR: Unable to Update Blueprints")
                    result = requests.patch(HOST + url, data=json.dumps(blueprint), headers=headers, cookies=session)
                    if result.status_code != 200:
                        print("ERROR: Updating blueprint failed for machine: " + machineName +", invalid blueprint config....")
                        sys.exit(4)
                    machinelist[blueprint["machineId"]] = "updated"
                    print("Blueprint for machine: " + machineName + " updated....")              
    except:
        print("ERROR: Updating blueprint task failed....")
        print(sys.exc_info())
        sys.exit(3)

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))